@@ -25,28 +25,32 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
+
 #include <algorithm>
 #include <atomic>
 #include <deque>
-#include <new>
+#include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "absl/base/attributes.h"
 #include "absl/base/call_once.h"
-#include "absl/base/macros.h"
 #include "absl/base/thread_annotations.h"
 #include "absl/container/flat_hash_map.h"
 #include "absl/container/flat_hash_set.h"
+#include "absl/hash/hash.h"
+#include "absl/log/absl_check.h"
+#include "absl/log/absl_log.h"
 #include "absl/strings/str_format.h"
+#include "absl/strings/string_view.h"
 #include "absl/synchronization/mutex.h"
 #include "absl/types/span.h"
-#include "util/logging.h"
-#include "util/strutil.h"
 #include "re2/pod_array.h"
 #include "re2/prog.h"
 #include "re2/re2.h"
 #include "re2/sparse_set.h"
+#include "util/strutil.h"
 
 // Silence "zero-sized array in struct/union" warning for DFA::State::next_.
 #ifdef _MSC_VER
@@ -134,15 +138,8 @@ class DFA {
                         // into this state, along with kFlagMatch if this
                         // is a matching state.
 
-// Work around the bug affecting flexible array members in GCC 6.x (for x >= 1).
-// (https://gcc.gnu.org/bugzilla/show_bug.cgi?id=70932)
-#if !defined(__clang__) && defined(__GNUC__) && __GNUC__ == 6 && __GNUC_MINOR__ >= 1
-    std::atomic<State*> next_[0];   // Outgoing arrows from State,
-                                    // one per input byte class
-#else
     std::atomic<State*> next_[];    // Outgoing arrows from State,
                                     // one per input byte class
-#endif
   };
 
   enum {
@@ -156,15 +153,15 @@ class DFA {
 
   struct StateHash {
     size_t operator()(const State* a) const {
-      DCHECK(a != NULL);
+      ABSL_DCHECK(a != NULL);
       return absl::Hash<State>()(*a);
     }
   };
 
   struct StateEqual {
     bool operator()(const State* a, const State* b) const {
-      DCHECK(a != NULL);
-      DCHECK(b != NULL);
+      ABSL_DCHECK(a != NULL);
+      ABSL_DCHECK(b != NULL);
       return *a == *b;
     }
   };
@@ -666,7 +663,7 @@ DFA::State* DFA::WorkqToCachedState(Workq* q, Workq* mq, uint32_t flag) {
         break;
     }
   }
-  DCHECK_LE(n, q->size());
+  ABSL_DCHECK_LE(n, q->size());
   if (n > 0 && inst[n-1] == Mark)
     n--;
 
@@ -854,7 +851,7 @@ void DFA::AddToQueue(Workq* q, int id, uint32_t flag) {
 
   stk[nstk++] = id;
   while (nstk > 0) {
-    DCHECK_LE(nstk, stack_.size());
+    ABSL_DCHECK_LE(nstk, stack_.size());
     id = stk[--nstk];
 
   Loop:
@@ -879,7 +876,7 @@ void DFA::AddToQueue(Workq* q, int id, uint32_t flag) {
     Prog::Inst* ip = prog_->inst(id);
     switch (ip->opcode()) {
       default:
-        LOG(DFATAL) << "unhandled opcode: " << ip->opcode();
+        ABSL_LOG(DFATAL) << "unhandled opcode: " << ip->opcode();
         break;
 
       case kInstByteRange:  // just save these on the queue
@@ -905,7 +902,7 @@ void DFA::AddToQueue(Workq* q, int id, uint32_t flag) {
         goto Loop;
 
       case kInstAltMatch:
-        DCHECK(!ip->last());
+        ABSL_DCHECK(!ip->last());
         id = id+1;
         goto Loop;
 
@@ -968,7 +965,7 @@ void DFA::RunWorkqOnByte(Workq* oldq, Workq* newq,
     Prog::Inst* ip = prog_->inst(id);
     switch (ip->opcode()) {
       default:
-        LOG(DFATAL) << "unhandled opcode: " << ip->opcode();
+        ABSL_LOG(DFATAL) << "unhandled opcode: " << ip->opcode();
         break;
 
       case kInstFail:        // never succeeds
@@ -1036,14 +1033,14 @@ DFA::State* DFA::RunStateOnByte(State* state, int c) {
       return FullMatchState;
     }
     if (state == DeadState) {
-      LOG(DFATAL) << "DeadState in RunStateOnByte";
+      ABSL_LOG(DFATAL) << "DeadState in RunStateOnByte";
       return NULL;
     }
     if (state == NULL) {
-      LOG(DFATAL) << "NULL state in RunStateOnByte";
+      ABSL_LOG(DFATAL) << "NULL state in RunStateOnByte";
       return NULL;
     }
-    LOG(DFATAL) << "Unexpected special state in RunStateOnByte";
+    ABSL_LOG(DFATAL) << "Unexpected special state in RunStateOnByte";
     return NULL;
   }
 
@@ -1274,7 +1271,7 @@ DFA::State* DFA::StateSaver::Restore() {
   absl::MutexLock l(&dfa_->mutex_);
   State* s = dfa_->CachedState(inst_, ninst_, flag_);
   if (s == NULL)
-    LOG(DFATAL) << "StateSaver failed to restore state.";
+    ABSL_LOG(DFATAL) << "StateSaver failed to restore state.";
   return s;
 }
 
@@ -1374,7 +1371,7 @@ inline bool DFA::InlinedSearchLoop(SearchParams* params) {
     lastmatch = p;
     if (ExtraDebug)
       absl::FPrintF(stderr, "match @stx! [%s]\n", DumpState(s));
-    if (params->matches != NULL && kind_ == Prog::kManyMatch) {
+    if (params->matches != NULL) {
       for (int i = s->ninst_ - 1; i >= 0; i--) {
         int id = s->inst_[i];
         if (id == MatchSep)
@@ -1458,13 +1455,13 @@ inline bool DFA::InlinedSearchLoop(SearchParams* params) {
         // Restore start and s so we can continue.
         if ((start = save_start.Restore()) == NULL ||
             (s = save_s.Restore()) == NULL) {
-          // Restore already did LOG(DFATAL).
+          // Restore already did ABSL_LOG(DFATAL).
           params->failed = true;
           return false;
         }
         ns = RunStateOnByteUnlocked(s, c);
         if (ns == NULL) {
-          LOG(DFATAL) << "RunStateOnByteUnlocked failed after ResetCache";
+          ABSL_LOG(DFATAL) << "RunStateOnByteUnlocked failed after ResetCache";
           params->failed = true;
           return false;
         }
@@ -1491,7 +1488,7 @@ inline bool DFA::InlinedSearchLoop(SearchParams* params) {
         lastmatch = p + 1;
       if (ExtraDebug)
         absl::FPrintF(stderr, "match @%d! [%s]\n", lastmatch - bp, DumpState(s));
-      if (params->matches != NULL && kind_ == Prog::kManyMatch) {
+      if (params->matches != NULL) {
         for (int i = s->ninst_ - 1; i >= 0; i--) {
           int id = s->inst_[i];
           if (id == MatchSep)
@@ -1536,7 +1533,7 @@ inline bool DFA::InlinedSearchLoop(SearchParams* params) {
       }
       ns = RunStateOnByteUnlocked(s, lastbyte);
       if (ns == NULL) {
-        LOG(DFATAL) << "RunStateOnByteUnlocked failed after Reset";
+        ABSL_LOG(DFATAL) << "RunStateOnByteUnlocked failed after Reset";
         params->failed = true;
         return false;
       }
@@ -1558,7 +1555,7 @@ inline bool DFA::InlinedSearchLoop(SearchParams* params) {
     lastmatch = p;
     if (ExtraDebug)
       absl::FPrintF(stderr, "match @etx! [%s]\n", DumpState(s));
-    if (params->matches != NULL && kind_ == Prog::kManyMatch) {
+    if (params->matches != NULL) {
       for (int i = s->ninst_ - 1; i >= 0; i--) {
         int id = s->inst_[i];
         if (id == MatchSep)
@@ -1653,7 +1650,7 @@ bool DFA::AnalyzeSearch(SearchParams* params) {
 
   // Sanity check: make sure that text lies within context.
   if (BeginPtr(text) < BeginPtr(context) || EndPtr(text) > EndPtr(context)) {
-    LOG(DFATAL) << "context does not contain text";
+    ABSL_LOG(DFATAL) << "context does not contain text";
     params->start = DeadState;
     return true;
   }
@@ -1701,7 +1698,7 @@ bool DFA::AnalyzeSearch(SearchParams* params) {
     ResetCache(params->cache_lock);
     if (!AnalyzeSearchHelper(params, info, flags)) {
       params->failed = true;
-      LOG(DFATAL) << "Failed to analyze start state.";
+      ABSL_LOG(DFATAL) << "Failed to analyze start state.";
       return false;
     }
   }
@@ -1774,6 +1771,8 @@ bool DFA::Search(absl::string_view text, absl::string_view context,
   params.anchored = anchored;
   params.want_earliest_match = want_earliest_match;
   params.run_forward = run_forward;
+  // matches should be null except when using RE2::Set.
+  ABSL_DCHECK(matches == NULL || kind_ == Prog::kManyMatch);
   params.matches = matches;
 
   if (!AnalyzeSearch(&params)) {
