@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
-#include "util/test.h"
+#include "absl/log/absl_log.h"
 #include "re2/regexp.h"
 #include "re2/walker-inl.h"
 
@@ -12,18 +12,22 @@ namespace re2 {
 
 class NullWalker : public Regexp::Walker<bool> {
  public:
-  NullWalker() { }
-  bool PostVisit(Regexp* re, bool parent_arg, bool pre_arg,
-                 bool* child_args, int nchild_args);
+  NullWalker() {}
 
-  bool ShortVisit(Regexp* re, bool a) {
-    // Should never be called: we use Walk not WalkExponential.
-    LOG(DFATAL) << "NullWalker::ShortVisit called";
+  virtual bool PostVisit(Regexp* re, bool parent_arg, bool pre_arg,
+                         bool* child_args, int nchild_args);
+
+  virtual bool ShortVisit(Regexp* re, bool a) {
+    // Should never be called: we use Walk(), not WalkExponential().
+#ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+    ABSL_LOG(DFATAL) << "NullWalker::ShortVisit called";
+#endif
     return a;
   }
 
  private:
-  DISALLOW_COPY_AND_ASSIGN(NullWalker);
+  NullWalker(const NullWalker&) = delete;
+  NullWalker& operator=(const NullWalker&) = delete;
 };
 
 // Called after visiting re's children.  child_args contains the return
